/* PulseAudioClipTest.java
   Copyright (C) 2008 Red Hat, Inc.

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 2.

IcedTea is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.classpath.icedtea.pulseaudio;

import java.io.File;
import java.io.IOException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.Control;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.UnsupportedAudioFileException;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class PulseAudioClipTest {

	Mixer mixer;
	AudioFormat aSupportedFormat = new AudioFormat(
			AudioFormat.Encoding.PCM_UNSIGNED, 44100f, 8, 1, 1, 44100f, true);

	@Before
	public void setUp() throws LineUnavailableException {
		Mixer.Info wantedMixerInfo = null;
		Mixer.Info[] mixerInfos = AudioSystem.getMixerInfo();
		for (Mixer.Info mixerInfo : mixerInfos) {
			if (mixerInfo.getName().contains("PulseAudio")) {
				wantedMixerInfo = mixerInfo;
				break;
			}
		}
		assert (wantedMixerInfo != null);
		mixer = AudioSystem.getMixer(wantedMixerInfo);
		mixer.open();
	}

	@Test
	public void testObtainingAClip() throws LineUnavailableException {
		System.out
				.println("This tests if a clip can be obtained from the mixer");
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		Assert.assertNotNull(clip);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testClipOpenWrongUse() throws LineUnavailableException {
		System.out.println("This test checks ");
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		clip.open();
	}

	@Test
	public void testClipOpens() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream = AudioSystem
				.getAudioInputStream(soundFile);
		clip.open(audioInputStream);
		clip.close();
	}

	@Test
	public void testLoopStopStartClip() throws LineUnavailableException,
			IOException, UnsupportedAudioFileException {
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream = AudioSystem
				.getAudioInputStream(soundFile);
		clip.open(audioInputStream);

		clip.setLoopPoints((int) (clip.getFrameLength() / 4), (int) (clip
				.getFrameLength() / 2));
		clip.loop(4);
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		clip.stop();
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		clip.start();
		clip.close();
	}

	@Test
	public void testIsActiveAndIsOpen() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {

		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream = AudioSystem
				.getAudioInputStream(soundFile);

		Assert.assertFalse(clip.isActive());
		Assert.assertFalse(clip.isOpen());
		clip.open(audioInputStream);
		Assert.assertTrue(clip.isOpen());
		Assert.assertFalse(clip.isActive());
		clip.start();
		Assert.assertTrue(clip.isOpen());
		Assert.assertTrue(clip.isActive());
		clip.stop();
		Assert.assertTrue(clip.isOpen());
		Assert.assertFalse(clip.isActive());
		clip.close();
		Assert.assertFalse(clip.isOpen());
		Assert.assertFalse(clip.isActive());

	}

	int opened = 0;

	@Test
	public void testOpenEvent() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream = AudioSystem
				.getAudioInputStream(soundFile);

		opened = 0;

		LineListener openListener = new LineListener() {

			@Override
			public void update(LineEvent event) {
				if (event.getType() == LineEvent.Type.OPEN) {
					opened++;
				}
			}

		};

		clip.addLineListener(openListener);
		clip.open(audioInputStream);
		clip.close();
		clip.removeLineListener(openListener);

		Assert.assertEquals(1, opened);

	}

	int closed = 0;

	@Test
	public void testCloseEvent() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {

		System.out.println("This tests the CLOSE event");

		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream = AudioSystem
				.getAudioInputStream(soundFile);

		closed = 0;

		LineListener closeListener = new LineListener() {

			@Override
			public void update(LineEvent event) {
				if (event.getType() == LineEvent.Type.CLOSE) {
					closed++;
				}
			}

		};

		clip.addLineListener(closeListener);
		clip.open(audioInputStream);
		clip.close();

		clip.removeLineListener(closeListener);

		Assert.assertEquals(1, closed);

	}

	int started = 0;
	int stopped = 0;

	@Test
	public void testStartedStopped() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {

		File soundFile = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream = AudioSystem
				.getAudioInputStream(soundFile);
		AudioFormat audioFormat = audioInputStream.getFormat();

		Clip clip;
		clip = (Clip) mixer.getLine(new DataLine.Info(Clip.class, audioFormat));
		Assert.assertNotNull(clip);

		started = 0;
		stopped = 0;

		clip.open(audioInputStream);

		LineListener startStopListener = new LineListener() {

			@Override
			public void update(LineEvent event) {
				if (event.getType() == LineEvent.Type.START) {
					started++;
					Assert.assertEquals(1, started);
				}

				if (event.getType() == LineEvent.Type.STOP) {
					System.out.println("Stopped event");
					stopped++;
					Assert.assertEquals(1, stopped);
				}
			}

		};

		clip.addLineListener(startStopListener);

		clip.start();
		clip.drain();

		clip.stop();
		clip.close();

		Assert.assertEquals(1, started);
		Assert.assertEquals(1, stopped);

		started = 0;
		stopped = 0;

	}

	@Test
	public void testLoop0Clip() throws LineUnavailableException, IOException,
			UnsupportedAudioFileException {
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream = AudioSystem
				.getAudioInputStream(soundFile);
		clip.open(audioInputStream);

		clip.setLoopPoints((int) (clip.getFrameLength() / 4), (int) (clip
				.getFrameLength() / 2));
		clip.loop(4);
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		clip.loop(0);
		clip.close();
	}

	@Test(expected = IllegalArgumentException.class)
	public void testOpenWrongUse() throws LineUnavailableException {
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		clip.open();

	}

	/*
	 * 
	 * modified version of the sample code at
	 * http://icedtea.classpath.org/bugzilla/show_bug.cgi?id=173
	 * 
	 */

	@Test
	public void testPlayTwoClips() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {
		Clip clip1 = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile1 = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream1 = AudioSystem
				.getAudioInputStream(soundFile1);
		clip1.open(audioInputStream1);

		Clip clip2 = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile2 = new File("testsounds/logout.wav");
		AudioInputStream audioInputStream2 = AudioSystem
				.getAudioInputStream(soundFile2);
		clip2.open(audioInputStream2);

		clip1.start();
		clip2.start();

		clip1.drain();
		clip2.drain();

		clip1.close();
		clip2.close();

		Assert.assertFalse(clip1.isOpen());
		Assert.assertFalse(clip2.isOpen());

	}

	@Test
	public void testSupportedControls() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile1 = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream1 = AudioSystem
				.getAudioInputStream(soundFile1);
		clip.open(audioInputStream1);

		Control[] controls = clip.getControls();
		Assert.assertNotNull(controls);
		Assert.assertTrue(controls.length >= 2);
		for (Control control : controls) {
			Assert.assertNotNull(control);
		}

		clip.close();
	}

	@Test
	public void testMixerKnowsAboutOpenClips() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {
		Clip clip = (Clip) mixer.getLine(new Line.Info(Clip.class));
		File soundFile1 = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream1 = AudioSystem
				.getAudioInputStream(soundFile1);

		Assert.assertEquals(0, mixer.getSourceLines().length);
		clip.open(audioInputStream1);
		Assert.assertEquals(1, mixer.getSourceLines().length);
		Assert.assertEquals(clip, mixer.getSourceLines()[0]);
		clip.close();
		Assert.assertEquals(0, mixer.getSourceLines().length);

	}

	@After
	public void tearDown() {
		mixer.close();
	}

}
